/* Copyright 2000-2013 Matt Flax <flatmax@flatmax.org>
   This file is part of GTK+ IOStream class set

   GTK+ IOStream is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   GTK+ IOStream is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You have received a copy of the GNU General Public License
   along with GTK+ IOStream
 */
#ifndef BUTTONS_H_
#define BUTTONS_H_

#include <iomanip>
#include <gtk/gtk.h>
#include <mffm/LinkList.H>

#ifdef _MSC_VER
#pragma warning( disable : 4005 4554)
#endif

/** \brief class to pass using operator<< for creating a label button
*/
class labelFuncData {
public:
#ifdef WIN32 // visual C++ doesn't support compound literals
	labelFuncData(void){str=NULL; func=NULL; data=NULL;}
	labelFuncData(char *s, void (*f)(void *, void *), gpointer d){str=s; func=f; data=d;}
#endif
	char *str; ///< The label to show in the button
    void (*func)(void *arg1, void *arg2); ///< The function to call when the button is pressed
    gpointer data;  ///< The data to pass to func when the button it pressed
};
#define BUTTONLABELSTRUCT(s,f,d) (labelFuncData){s,f,d}
#ifdef WIN32 // visual C++ doesn't support compound literals
#define BUTTONLABELSTRUCT(s,f,d) labelFuncData(s,f,d)
#endif

/// Used to create a PIXMAP for the button
#define GDKPIXMAPCREATE(win, ref) gdk_pixmap_colormap_create_from_xpm_d(win->window,gtk_widget_get_colormap(win),NULL,NULL,ref)
/// Used to create a PIXMAP for the button
#define GTKPIXMAPCREATE(pixGdk) (GtkPixmap*)gtk_image_new_from_pixmap(pixGdk, NULL)
/** \brief class to pass using operator<< for creating a regular pixmap on a button
*/
class pixmapFuncData {
public:
#ifdef WIN32 // visual C++ doesn't support compound literals
	pixmapFuncData(const char **p, void (*f)(void *, void *), gpointer d){pixGtk=p; func=f; data=d;}
#endif
    const char **pixGtk; ///< The pixmap to show
    void (*func)(void *arg1, void *arg2); ///< The function to call when the button is pressed
    gpointer data;  ///< The data to pass to func when the button it pressed
};
#define BUTTONPIXMAPSTRUCT(p,f,d) (pixmapFuncData){p,f,d}
#ifdef WIN32 // visual C++ doesn't support compound literals
#define BUTTONPIXMAPSTRUCT(p,f,d) pixmapFuncData(p,f,d)
#endif

/** \brief class to pass for creating a transparent XPM overlay on a button
*/
class pixmapFuncDataTransparent {
public:
#ifdef WIN32 // visual C++ doesn't support compound literals
	pixmapFuncDataTransparent(const char **i, void (*f)(void *, void *), gpointer d){imageXPM=i; func=f; data=d;}
#endif
    const char **imageXPM; ///< The pointer to the image for overlaying on the button
    void (*func)(void *arg1, void *arg2); ///< The function to call when the button is pressed
    gpointer data; ///< The data to pass to func when the button it pressed
};
#define BUTTONPIXMAPSTRUCTTRANSPARENT(i,f,d) (pixmapFuncDataTransparent){i,f,d}
#ifdef WIN32 // visual C++ doesn't support compound literals
#define BUTTONPIXMAPSTRUCTTRANSPARENT(i,f,d) pixmapFuncDataTransparent(i,f,d)
#endif

/** \brief class to pass using operator<< for creating a toggle button with a label
*/
class toggleFuncData {
public:
#ifdef WIN32 // visual C++ doesn't support compound literals
	toggleFuncData(char *s, void (*f)(void *, void *), gpointer d){str=s; func=f; data=d;}
#endif
	char *str; ///< The label to display on the button
    void (*func)(void *arg1, void *arg2); ///< The function to call when the button is pressed
    gpointer data;  ///< The data to pass to func when the button it pressed
};
#define BUTTONTOGGLESTRUCT(s,f,d) (toggleFuncData){s,f,d}
#ifdef WIN32 // visual C++ doesn't support compound literals
#define BUTTONTOGGLESTRUCT(s,f,d) toggleFuncData(s,f,d)
#endif

/** \brief class to pass using operator<< for creating an arrow button
*/
class arrowFuncData {
public:
#ifdef WIN32 // visual C++ doesn't support compound literals
	arrowFuncData(void){func=NULL; data=NULL; whichArrow=0; whichShadow=0;}
	arrowFuncData(void (*f)(void *, void *), gpointer d, GtkArrowType at, GtkShadowType sh, GtkWidget *a){func=f; data=d; whichArrow=at; whichShadow=sh;}
#endif
    void (*func)(void *arg1, void *arg2); ///< The function to call when the button is pressed
    gpointer data;  ///< The data to pass to func when the button it pressed
    GtkArrowType whichArrow; ///< The type of arrow to create {GTK_ARROW_UP, GTK_ARROW_DOWN, GTK_ARROW_LEFT, GTK_ARROW_RIGHT, GTK_ARROW_NONE}
    GtkShadowType whichShadow; ///< The shadow to use {GTK_SHADOW_NONE, GTK_SHADOW_IN, GTK_SHADOW_OUT, GTK_SHADOW_ETCHED_IN, GTK_SHADOW_ETCHED_OUT}
};
#define BUTTONARROWSTRUCT(f, d, a, sh) (arrowFuncData){f,d, a, sh}
#ifdef WIN32 // visual C++ doesn't support compound literals
#define BUTTONARROWSTRUCT(f,d,a,sh) arrowFuncData(f,d,a,sh)
#endif

/** \brief class to pass using operator<< for creating a radio button with a label
*/
class radioFuncData {
public:
#ifdef WIN32 // visual C++ doesn't support compound literals
	radioFuncData(char *s, void (*f)(void *, void *), gpointer d){str=s; func=f; data=d;}
#endif
	char *str; ///< The label to display on the button
    void (*func)(void *arg1, void *arg2); ///< The function to call when the button is pressed
    gpointer data;  ///< The data to pass to func when the button it pressed
};
#define BUTTONRADIOSTRUCT(s,f,d) (radioFuncData){s,f,d}
#ifdef WIN32 // visual C++ doesn't support compound literals
#define BUTTONRADIOSTRUCT(s,f,d) radioFuncData(s,f,d)
#endif

/** \brief Class to setup buttons whic are either text, image or toggle types.
Buttons are defined in terms of their callbacks functions and data as well as their type (image, text or toggle)
You stream in any new buttons you want to create, for example :
\code
Buttons buttons << BUTTONLABELSTRUCT {"display text", callbackFunc, (void *)dataVariable};
\endcode
This will create a button with the display text, when pressed, the callbackFunc will be called with the dataVariable as a variable.
You can add buttons to your window like so :
\code
// create 2 buttons
Buttons buttons << BUTTONLABELSTRUCT {"button1", callbackFunc1, (void *)dataVariable1} << BUTTONLABELSTRUCT {"button2", callbackFunc2, (void *)dataVariable2};
HBox box; // A horizontal box to show the buttons in
for (int i=0;i<buttons.getCount();i++) // add the buttons to the box
    box << buttons.next();
box.show(); // make sure the box is not hidden
\endcode
*/
class Buttons : public LinkList<GtkWidget *> {
public:

    /** \brief Constructor
    */
    Buttons(void) {
        gdk_colormap_new(gdk_visual_get_best(),1);
    }

    /** Set the alignment within the button.
    Operates on the GtkWidget* which is LinkList::current - i.e. currently pointed to
    \param xAlign the x-alignment 0.0 is a left alignment, 1.0 is for right alignment
    \param yAlign the y-alignment 0.0 is a left alignment, 1.0 is for right alignment
    */
    void setAlignment(float xAlign, float yAlign){
         gtk_button_set_alignment(GTK_BUTTON(current()), xAlign, yAlign);
         gtk_widget_queue_draw(current());
    }

    /** Get the alignment within the button.
    Operates on the GtkWidget* which is LinkList::current - i.e. currently pointed to
    Fills the pointers to the floats with the alignment settings.
    \param xAlign the x-alignment 0.0 is a left alignment, 1.0 is for right alignment
    \param yAlign the y-alignment 0.0 is a left alignment, 1.0 is for right alignment
    */
    void getAlignment(float *xAlign, float *yAlign){
         gtk_button_get_alignment(GTK_BUTTON(current()), xAlign, yAlign);
    }

//    /** \brief Destructor
//    */
//    ~Buttons(void){
//        for (int i=1; i<=getCount(); i++)
//            if (g_object_is_floating(current()))
//                delete remove();
//    }
//
    /** \brief connect a callback to execute after the currently connected callbacks
    \param nf labelFuncData matching the callback to connect
    */
    void signalConnectAfter(labelFuncData nf) {
        if (nf.func)
            g_signal_connect_after(GTK_OBJECT((GtkButton*)current()), "released",GTK_SIGNAL_FUNC(nf.func), nf.data);
    }

    /** \brief disconnect the old callback and reconnect a new callback
    \param nfOld labelFuncData matching the old callback to be removed
    \param nfNew labelFuncData matching the new to be connected callback
    */
    void signalReplace(labelFuncData nfOld, labelFuncData nfNew) {
        if (nfOld.func)
            g_signal_handlers_disconnect_by_func(GTK_OBJECT((GtkButton*)current()), (void *)(nfOld.func), nfOld.data);
        if (nfNew.func)
            g_signal_connect(GTK_OBJECT((GtkButton*)current()), "released",GTK_SIGNAL_FUNC(nfNew.func), nfNew.data);
    }


    /** \brief disconnect the old callback and reconnect a new callback
    \param nfOld labelFuncData matching the old callback to be removed
    \param nfNew labelFuncData matching the new to be connected callback
    */
    void signalRemove(labelFuncData nfOld) {
        if (nfOld.func){
            g_signal_handlers_disconnect_by_func(GTK_OBJECT((GtkButton*)current()), (void *)(nfOld.func), nfOld.data);
        }
    }

    /** \brief generate a label button
    \param labelFuncData The label, callback function and data to show.
    \return This Buttons class for further overloading
    */
    Buttons& operator <<(labelFuncData nf) {
        if (nf.str)
            LinkList<GtkWidget *>::add(gtk_button_new_with_label(nf.str));
        else
            LinkList<GtkWidget *>::add(gtk_button_new());

        if (nf.func)
            g_signal_connect(GTK_OBJECT((GtkButton*)current()), "released",GTK_SIGNAL_FUNC(nf.func), nf.data);
        gtk_widget_show(current());
        return *this;
    }

    /** \brief generate a button with a static pixmap
    \param pixmapFuncData The pixmap, callback function and data to show.
    \return This Buttons class for further overloading
    */
    Buttons& operator <<(const pixmapFuncData nf) {
//        GdkPixbuf * pixBuf=gdk_pixbuf_new_from_xpm_data(nf.pixGtk);
        LinkList<GtkWidget *>::add(gtk_button_new());
//        gtk_widget_set_size_request(current(), gdk_pixbuf_get_width(pixBuf), gdk_pixbuf_get_height(pixBuf)); // resize the button to match  the pixmap
//        cairo_t  *cr=gdk_cairo_create(current()->window); // get a cairo context for rendering
//        gdk_cairo_set_source_pixbuf(cr, pixBuf,  current()->allocation.x,  current()->allocation.y); // set the source to the pixbuf at the correct location
//        cairo_paint(cr); // paint it to the cairo context
//        cairo_destroy(cr); // destroy the context
        g_signal_connect_after(current(), "expose-event", G_CALLBACK(exposePixmap), (gpointer)nf.pixGtk);

        if (nf.func)
            gtk_signal_connect(GTK_OBJECT(current()), "released",GTK_SIGNAL_FUNC(nf.func), nf.data);
        gtk_widget_show(current());
        return *this;
    }

    /** \brief generate a toggle button with label
    \param toggleFuncData The label, callback function and data to show.
    \return This Buttons class for further overloading
    */
    Buttons& operator <<(toggleFuncData nf) {
        LinkList<GtkWidget *>::add(gtk_toggle_button_new_with_label(nf.str));
        if (nf.func)
            gtk_signal_connect(GTK_OBJECT((GtkButton*)current()), "released",GTK_SIGNAL_FUNC(nf.func), nf.data);
        gtk_widget_show(current());
        return *this;
    }

    /** \brief generate a button with a pixmap which is partially transparent
    \param pixmapFuncDataTransparent The pixmap, callback function and data to show.
    \return This Buttons class for further overloading
    */
    Buttons& operator <<(const pixmapFuncDataTransparent pfdt) {
        LinkList<GtkWidget *>::add(gtk_button_new());

        g_signal_connect_after(current(), "expose-event", G_CALLBACK(exposePixmap), (gpointer)pfdt.imageXPM);

        if (pfdt.func)
            g_signal_connect(GTK_OBJECT(current()), "released",GTK_SIGNAL_FUNC(pfdt.func), pfdt.data);
        gtk_widget_show(current());
        return *this;
    }

    /** \brief generate an arrow button
    If the arrow widget is supplied in the arrowFuncData class, then it will be used instead of generating a new one.
    See arroFuncData for the various class member variables which are used in this method.
    \param arrowFuncData The callback function, data, arrow type, shadow type and possibly a pre-existing arrow to use/show. If the pre-existing arrow is to be used, then the arrowFuncData.whichArrow and arrowFuncData.whichShdow variables are ignored.
    \return This Buttons class for further overloading
    */
    Buttons& operator <<(arrowFuncData af) {
        LinkList<GtkWidget *>::add(gtk_button_new());
        GtkWidget *arrow=gtk_arrow_new(af.whichArrow,af.whichShadow);
        gtk_container_add (GTK_CONTAINER (current()), arrow);
        gtk_widget_show(arrow);

        if (af.func)
            gtk_signal_connect(GTK_OBJECT((GtkButton*)current()), "released",GTK_SIGNAL_FUNC(af.func), af.data);
        gtk_widget_show_all(current());
        return *this;
    }

    /** \brief generate a radio button with label
    If other radio buttons are present, then they are all added to the same group
    \param radioFuncData The label, callback function and data to show.
    \return This Buttons class for further overloading
    */
    Buttons& operator <<(radioFuncData nf) {
        if (!LinkList<GtkWidget *>::getCount()) // this is the first
            LinkList<GtkWidget *>::add(gtk_radio_button_new_with_label(NULL,nf.str));
        else
            LinkList<GtkWidget *>::add(gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (current()), nf.str));
        if (nf.func)
            gtk_signal_connect(GTK_OBJECT((GtkButton*)current()), "released",GTK_SIGNAL_FUNC(nf.func), nf.data);
        gtk_widget_show(current());
        return *this;
    }

    /** \brief overlay the pixmap image onto the button whenever required
    This method allows transparent XPMs to be displayed on buttons.
    Whenever the background is changed, the transparent pixmap is re-drawn
    \brief widget The button to re-draw
    \brief event the event (we are looking for an expose event)
    \brief data The XPM pointer (char**) to use for display
    \return false to keep propagating the event
    */
    static gboolean exposePixmap(GtkWidget *widget, GdkEventExpose *event, gpointer data) {
        GdkPixbuf *tipGDKPixBuf = gdk_pixbuf_new_from_xpm_data(static_cast<const char**>(data)); // get the pixbuf
        int width = gdk_pixbuf_get_width(tipGDKPixBuf), height = gdk_pixbuf_get_height(tipGDKPixBuf);
        if ((widget->allocation.width<width) | (widget->allocation.height<height))
            gtk_widget_set_size_request(widget, width, height); // resize the button to match  the pixmap
        else {
            cairo_t  *cr=gdk_cairo_create(widget->window); // get a cairo context for rendering

            // use alignment to draw appropriately
            float xAlign, yAlign;
            gtk_button_get_alignment(GTK_BUTTON(widget), &xAlign, &yAlign);
            int xOffset=(int)((float)(widget->allocation.width-width-1)*xAlign); if (xOffset<0) xOffset=0;
            int yOffset=(int)((float)(widget->allocation.height-height-1)*yAlign); if (yOffset<0) yOffset=0;

            gdk_cairo_set_source_pixbuf(cr, tipGDKPixBuf,  widget->allocation.x+xOffset,  widget->allocation.y+yOffset); // set the source to the pixbuf at the correct location
            cairo_paint(cr); // paint it to the cairo context
            cairo_destroy(cr); // destroy the context
        }
        return FALSE;
    }

    /** Set the current botton's label's font
    For example :
    \code
    #include "Pango.H"

    Font font("bold"); // could be something else, like "Sans 12"

    Buttons button<<BUTTONLABELSTRUCT("I want this to be bold", (void (*)(void *, void *))buttonPressed, (void *)this);
    button.setLabelsFont(font.getPangoFontDescription());
    \endcode
    */
    void setLabelsFont(PangoFontDescription *pangoFontDescription){
        if (pangoFontDescription){
            GtkWidget *child=gtk_bin_get_child(GTK_BIN(current()));
            if (child)
                gtk_widget_modify_font(child,pangoFontDescription);
        }
    }

    /** Find the GTK widget index (from 1) in the list of buttons
    \param widget The widget to look for
    \return 0 if it isn't on the list, the index (from 1) otherwise
    */
    int findIndex(GtkWidget* widget){
        int ret=0;
        if (getCount()){
            GtkWidget *c=current(); // remember our current location
            grab(1); prev(); // we are at the last on the list
            for (int i=1;i<=getCount();i++){
                if (next()==widget){
                    ret=i;
                    break;
                }
            }
            while (next()!=c) ; // return to the original button
        }
        //std::cout<<"Buttons::findIndex = "<<ret<<std::endl;
        return ret;
    }

    /** Get the widget at index (starting from 1)
    \param index the widget at this index to grab (starting from 1)
    \return The widget at index, or NULL on error
    */
    GtkWidget *findWidget(int index){
        GtkWidget *retVal=NULL;
        if (index<getCount() && index>0)
            retVal=grab(index);
        return retVal;
    }
};
#endif //BUTTONS_H_

