/* Copyright 2000-2013 Matt Flax <flatmax@flatmax.org>
   This file is part of GTK+ IOStream class set

   GTK+ IOStream is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   GTK+ IOStream is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You have received a copy of the GNU General Public License
   along with GTK+ IOStream
*/
#ifndef DRAWINGAREA_H_
#define DRAWINGAREA_H_

#include <gtk/gtk.h>
#include "Pixmap.H"

/** DrawingArea

This class allows the user to draw within a window by moving the mouse.
It uses a backing pixmap to store drawing strokes.
\image html DrawingAreaTest.jpg
*/
class DrawingArea : public Pixmap {

        /** Handler for when the button is pressed
        \param widget Which widget the button is pressed in.
        \param event The event to handle
        \return TRUE to continue processing the event
    */
    static gint button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer data) {
        DrawingArea *da=static_cast<DrawingArea*>(data);
        if (event->button == 1 && da->getPixmap() != NULL)
            da->draw_brush(widget, event->x, event->y);
        return TRUE;
    }

    /**
    */
    static gint motion_notify_event (GtkWidget *widget, GdkEventMotion *event, gpointer data) {
        DrawingArea *da=static_cast<DrawingArea*>(data);
        int x, y;
        GdkModifierType state;

        if (event->is_hint)
            gdk_window_get_pointer (event->window, &x, &y, &state);
        else {
            x = event->x;
            y = event->y;
            state = (GdkModifierType)event->state;
        }

        if (state & GDK_BUTTON1_MASK && da->getPixmap() != NULL)
            da->draw_brush (widget, x, y);

        return TRUE;
    }

    /** Create a valid drawing area of particular size
    \param width The width of the area
    \param height The height of the area
    */
    void init(void){
        g_signal_connect (GTK_OBJECT(widget), "motion_notify_event", (GtkSignalFunc) motion_notify_event, this);
        g_signal_connect (GTK_OBJECT(widget), "button_press_event", (GtkSignalFunc) button_press_event, this);

        gtk_widget_set_events (widget, GDK_EXPOSURE_MASK | GDK_LEAVE_NOTIFY_MASK | GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK);
    }

public:

    /** Constructor
    Create a drawing of default size (200 by 200)
    */
    DrawingArea(void) : Pixmap() {
        init();
    }

    /** Constructor
    Create a drawing area of particular size
    */
    DrawingArea(int width, int height) : Pixmap(width, height) {
        init();
    }

    /** Destructor
    */
    virtual ~DrawingArea(void) {
    }

    /** Draw a rectangle on the screen */
    void draw_brush(GtkWidget *widget, gdouble x, gdouble y) {
        GdkRectangle update_rect;

        update_rect.x = x - 5;
        update_rect.y = y - 5;
        update_rect.width = 10;
        update_rect.height = 10;
        gdk_draw_rectangle (pixmap, widget->style->white_gc, TRUE, update_rect.x, update_rect.y, update_rect.width, update_rect.height);
        gtk_widget_queue_draw_area (widget, update_rect.x, update_rect.y, update_rect.width, update_rect.height);
    }

};

#endif // DRAWINGAREA_H_
