/* Copyright 2000-2013 Matt Flax <flatmax@flatmax.org>
   This file is part of GTK+ IOStream class set

   GTK+ IOStream is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   GTK+ IOStream is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You have received a copy of the GNU General Public License
   along with GTK+ IOStream
*/

#include "Octave.H"
#include <iostream>

#include <octave/config.h>
#include <octave/octave.h>
#include <octave/oct-obj.h>
#include <octave/oct-map.h>

// taken from octave/toplev.h
typedef void (*octave_exit_func) (int);
extern OCTINTERP_API octave_exit_func octave_exit;

extern OCTINTERP_API octave_value_list
feval (const std::string& name, const octave_value_list& args = octave_value_list (), int nargout = 0);

Octave::Octave(const vector<string> &args) {
    int argc=args.size();
    vector<const char *> argsIn(argc);
    for (int i=0; i<argc; i++){
        argsIn[i]=&args[i][0];
        cout<<argsIn[i]<<endl;
    }
    int embedded=1;
    octave_main((int)argsIn.size(), (char**)&argsIn[0], embedded);
}

/// Destructor
Octave::~Octave() {
    octave_exit(0); // close the octave instance
}

template<class TYPE>
vector<Eigen::Matrix<TYPE, Eigen::Dynamic, Eigen::Dynamic> > Octave::runM(const char* commandName, vector<Eigen::Matrix<TYPE, Eigen::Dynamic, Eigen::Dynamic> > &in) {
    octave_value_list input; ///< Input variables to pass to Octave

    // this code can most likely be sped up using Eigen::Matrix.block operations for memory copies.
    input.resize(in.size());
    for (int i=0; i<in.size(); i++) {
        Matrix m(in[i].rows(),in[i].cols());
        for (int j=0; j<in[i].rows(); j++)
            for (int k=0; k<in[i].cols(); k++)
                m.elem(j,k)=(double)in[i](j,k);
        input(i)=m;
    }

    octave_value_list output; ///< Output variables returned from Octave
    //output = feval(string(commandName), input, retCount);
    output = feval(string(commandName), input);

    vector<Eigen::Matrix<TYPE, Eigen::Dynamic, Eigen::Dynamic> > out(output.length());
    for (int i=0; i<out.size(); i++) {
        Matrix m=output(i).matrix_value();
        Eigen::Matrix<TYPE, Eigen::Dynamic, Eigen::Dynamic> eigenM(m.rows(),m.columns());
        for (int j=0; j<m.rows(); j++)
            for (int k=0; k<m.columns(); k++)
                eigenM(j,k)=(TYPE)m.elem(j,k);
        out[i]=eigenM;
    }

    return out;
}
template vector<Eigen::Matrix<float, Eigen::Dynamic, Eigen::Dynamic> >  Octave::runM(const char* commandName, vector<Eigen::Matrix<float, Eigen::Dynamic, Eigen::Dynamic> > &in);
template vector<Eigen::Matrix<double, Eigen::Dynamic, Eigen::Dynamic> >  Octave::runM(const char* commandName, vector<Eigen::Matrix<double, Eigen::Dynamic, Eigen::Dynamic> > &in);
