/* Copyright 2000-2013 Matt Flax <flatmax@flatmax.org>
   This file is part of GTK+ IOStream class set

   GTK+ IOStream is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   GTK+ IOStream is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You have received a copy of the GNU General Public License
   along with GTK+ IOStream
 */
#ifndef PIXMAP_H_
#define PIXMAP_H_

#include <gtk/gtk.h>

class Pixmap {
    protected:
    GtkWidget *widget; ///< The drawing area widget
    GdkPixmap *pixmap; ///< Backing pixmap for drawing area

    /*! Method to show the box.
    \return box the GtkWidget* to the box
    */
    GtkWidget* show(void) {
        gtk_widget_show(widget);
        return widget;
    }
    /*! Method to hide the box.
    \return box the GtkWidget* to the box
    */
    GtkWidget* hide(void) {
        gtk_widget_hide(widget);
	return widget;
    }


    /** Create a new backing pixmap of the appropriate size */
    static gboolean configure_event( GtkWidget *widget, GdkEventConfigure *event, gpointer data) {
        Pixmap *da=static_cast<Pixmap*>(data);
        da->setPixmap(gdk_pixmap_new(widget->window, widget->allocation.width, widget->allocation.height, -1));
        gdk_draw_rectangle (da->getPixmap(), widget->style->black_gc, TRUE, 0, 0, widget->allocation.width, widget->allocation.height);

        return TRUE;
    }

    /** Redraw the screen from the backing pixmap */
    static gboolean expose_event( GtkWidget *widget, GdkEventExpose *event, gpointer data) {
        Pixmap *da=static_cast<Pixmap*>(data);
        gdk_draw_drawable(widget->window, widget->style->fg_gc[gtk_widget_get_state (widget)], da->getPixmap(), event->area.x, event->area.y, event->area.x, event->area.y,
                          event->area.width, event->area.height);

        return FALSE;
    }

    /** Method to dereference and set the pixmap to null */
    void destroyPixmap(void){
        if (pixmap){
            g_object_unref(pixmap);
            pixmap=NULL;
        }
    }

    void init(int width, int height){
                pixmap=NULL;
        widget=gtk_drawing_area_new ();
        show();

        g_signal_connect (GTK_OBJECT(widget), "expose_event", (GtkSignalFunc) expose_event, this);
        g_signal_connect (GTK_OBJECT(widget), "configure_event", (GtkSignalFunc) configure_event, this);

        gtk_drawing_area_size(GTK_DRAWING_AREA(widget), width, height);
    }
    public:

        /** Default constructor of size 200 by 200
        */
        Pixmap(){
            init(200, 200);
        }
        /** Construct a pixmap
        \param width the width of the pixmap
        \param height the height of the pixmap
        */
        Pixmap(int width, int height){
            init(width, height);
        }

            /** Destructor
    */
    virtual ~Pixmap(void) {
        destroyPixmap();
    }

    /** Get the drawing area widget
    \return The drawing area widget
    */
    GtkWidget *getWidget(void){return widget;}

    /** Get the backing pixmap
    \return The backing pixmap
    */
    GdkPixmap *getPixmap(void){return pixmap;}

    /** Set the backing pixmap
    \param The new backing pixmap
    */
    void setPixmap(GdkPixmap *pixmap_){
        destroyPixmap();
        pixmap=pixmap_;
    }


};
#endif  //PIXMAP_H_
