/* Copyright 2000-2013 Matt Flax <flatmax@flatmax.org>
   This file is part of GTK+ IOStream class set

   GTK+ IOStream is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   GTK+ IOStream is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You have received a copy of the GNU General Public License
   along with GTK+ IOStream
 */
#ifndef SCALE_H_
#define SCALE_H_

#include <iomanip>
#include <gtk/gtk.h>

class scaleFuncData {
public:
  int min, max, step;
  void (*func)(void *arg1, void *arg2);
  gpointer data;
};
#define SCALESTRUCT (scaleFuncData)

class Scale {
public:
  GtkWidget *scale;
public:
  GtkWidget* getWidget(void){return scale;}
  void show(void){gtk_widget_show(scale);}
  void hide(void){gtk_widget_hide(scale);}

  ~Scale(void){
    //    gtk_widget_destroy(scale);
    //#ifdef DEBUG
    //    cout<<"Scale::~Scale"<<endl;
    //#endif
  }

  double getCount(GtkRange * widget){
    return gtk_range_get_value(widget);
  }
  double getCount(void){
    return gtk_range_get_value((GtkRange*)scale);
  }

  Scale& operator =(double pos){
    gtk_range_set_value((GtkRange*)scale,pos);
    return *this;
  }
  Scale& operator =(int pos){
    return this->operator=((double)pos);
  }
};

class VScale : public Scale {
public:
  VScale(int min, int max, int step){
    scale = gtk_vscale_new_with_range(min,max,step);
  }

  VScale(scaleFuncData sf){
    scale = gtk_vscale_new_with_range(sf.min,sf.max,sf.step);
    if (sf.func){ //set the callback
      g_signal_connect(G_OBJECT((GtkScale*)scale), "value_changed",GTK_SIGNAL_FUNC(sf.func), sf.data);
    }
  }
};

class HScale  : public Scale {
public:
  HScale(int min, int max, int step){
    scale = gtk_hscale_new_with_range(min,max,step);
  }
  HScale(scaleFuncData sf){
    scale = gtk_hscale_new_with_range(sf.min,sf.max,sf.step);
    gtk_scale_set_draw_value((GtkScale*)scale, FALSE);
    if (sf.func){ //set the callback
      g_signal_connect(G_OBJECT((GtkScale*)scale), "value_changed",GTK_SIGNAL_FUNC(sf.func), sf.data);
    }
  }
};

#endif //SCALE_H_
