/* Copyright 2000-2013 Matt Flax <flatmax@flatmax.org>
   This file is part of GTK+ IOStream class set

   GTK+ IOStream is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   GTK+ IOStream is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You have received a copy of the GNU General Public License
   along with GTK+ IOStream
 */

#ifndef SCROLLING_H_
#define SCROLLING_H_

#include <gtk/gtk.h>
#include "X/Selection.H"

/** \brief Scrolling container
*/
class Scrolling {
protected:
    GtkWidget *scrolling; ///< The scrolling window
public:

    /** Construct a Scrolling widget
    */
    Scrolling(void) {
         scrolling = gtk_scrolled_window_new(NULL, NULL);
        gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolling), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    }

//    /** \brief change the callback data to be returned with the callback.
//    \param callback the callback function to be called upon selection change
//    \param data the data to be returned with the selection changed callback.
//    */
//    void setCallbackAfter(GCallback callback, void *data){
//        g_signal_connect_after(selection, "changed", G_CALLBACK(callback), data);
//    }

    /*! Gets the widget for loading into containers
    */
    GtkWidget *getWidget(void) {
        return scrolling;
    }

    /** A method to show
    */
    void show(void) {
        gtk_widget_show_all(scrolling);
    }
    /** A method to hide
    */
    void hide(void) {
        gtk_widget_hide(scrolling);
    }


    /** \brief add a selection object to the scrolling window
    \param selection the selection widget to scroll
    \return This Scrolling object
    */
    Scrolling & operator<<(Selection *selection) {
        gtk_container_add(GTK_CONTAINER(scrolling), selection->getWidget());
        /* Set vertical adjustment for the tree view */
        GtkAdjustment *vscroll = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scrolling));
        gtk_tree_view_set_vadjustment(GTK_TREE_VIEW (selection->getWidget()), vscroll);
        /* Set vertical adjustment for the tree view */
        GtkAdjustment *hscroll = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(scrolling));
        gtk_tree_view_set_hadjustment(GTK_TREE_VIEW (selection->getWidget()), hscroll);
        return *this;
    }

    /** \brief remove a widget from the scrolling window
    \param widget the widget to remove from the scrolling window
    \return This Scrolling object
    */
    Scrolling & operator>>(GtkWidget *widget) {
        gtk_container_remove(GTK_CONTAINER(scrolling), widget);
        return *this;
    }
};
#endif //SCROLLING_H_
