/* Copyright 2000-2013 Matt Flax <flatmax@flatmax.org>
   This file is part of GTK+ IOStream class set

   GTK+ IOStream is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   GTK+ IOStream is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You have received a copy of the GNU General Public License
   along with GTK+ IOStream
*/
#ifndef TEXTVIEW_H_
#define TEXTVIEW_H_

#include "Container.H"

/** TextView

Simple TextView widget.

Here is how to get the text from the text view:
\code
    GtkTextIter start, end;
    char *text_;
    GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (entry.getWidget()));
    gtk_text_buffer_get_bounds (buffer, &start, &end);
    text_ = gtk_text_iter_get_text (&start, &end);
\endcode

*/
class TextView : public Container {
public:
    /** Constructor
    */
    TextView(void){
        widget=gtk_text_view_new (); // create the entry box
    }

    /** Set the font
    For example :
    \code
    #include "Pango.H"

    Font font("bold"); // could be something else, like "Sans 12"

    TextView textView;
    textView.setLabelsFont(font.getPangoFontDescription());
    \endcode
    */
    void setFont(PangoFontDescription *pangoFontDescription){
        if (pangoFontDescription)
            gtk_widget_modify_font(widget,pangoFontDescription);
    }

    /** Connect a callback to capture when the text has changed in the text view  buffer
    */
    void connectBufferChangedSignal(GCallback callBack, void *data){
        //g_signal_connect(widget, "insert-at-cursor", callBack, data);
        GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(widget));
        g_signal_connect(GTK_WIDGET(buffer), "changed", callBack, data);
    }
};

#endif // TEXTVIEW_H_
